unit ViewFrame;

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Clipbrd,Dialogs,
  ExtCtrls,light, ImgList, StdCtrls, Spin, ComCtrls,TextImages,Math,types,convImages;

type

  TImageParts = class(TObject)
  public
   bitmap:TBitmap;
   x,y,h,w,alpha:Integer;
   mode:ShortInt;
   selected:boolean;
   constructor create();
   function getPosInf():TPosInf;
  end;

  PImageParts = ^TImageParts;
  ifMode = (ifNormal,ifmove,ifResizeAll,ifResizeV,ifResizeH,IfInit);

  TNotifyPartsEvent = procedure (index:Integer;parts:TImageParts) of object;
  TNotifyNameEvent = procedure (name:string) of object;

  TImageFrame = class(TFrame)
   Image1: TImage;
   procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
   procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
   procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FrameDblClick(Sender: TObject);

  private
    { Private 錾 }
    ImageWidth,ImageHeight:Integer;
    FNotifyPartsEvent:TNotifyPartsEvent;
    FNotifyNameEvent:TNotifyNameEvent;
    List:TStringList;
    selectedParts:TImageParts;
    mypoint:TPoint;
    selectID:Integer;
    selectRect:TRect;
    ADCanvas:TADCanvas;
    isInitImage:boolean;
    backBitmap,arrangedBitmap,initBitmap:TBitmap;

  public                                //   ,preSelectBitmap
    { Public 錾 }

    mode:ifMode;
    drawlock:boolean;
    constructor create(Owner:TComponent);override;
    procedure arrangeImage(inf:TImageInfo);
    procedure add(name:string;newParts:TImageParts);
    procedure delete(index:Integer);
    procedure exchange(index1,index2:Integer);
    procedure repaintParts();
    procedure readyInitImage();
    procedure setBackColor(const color:TColor);
    function getBackColor():TColor;
    procedure selectParts(index:Integer);
    procedure setLocation(inf:TPosInf);
    procedure resizeImage(w,h:Integer);
    procedure saveImage(fileName:string;saveType:byte=1);
    procedure drawSelected();
    procedure drawPartsEdge(parts:TImageParts);
    function IsOnRect(pt:TPoint;rec:TRect):Integer;
    function checkAllParts(pt:TPoint):Integer;
    procedure startMove(index:Integer);
    procedure reset;
    procedure setInitImage(bitmap:TBitmap);
    destructor destroy();override;
    function getLocation(index:Integer):TPosInf;
  published
    property NotifyPartsEvent:TNotifyPartsEvent
     read FNotifyPartsEvent write FNotifyPartsEvent;
    property NotifyNameEvent:TNotifyNameEvent
     read FNotifyNameEvent write FNotifyNameEvent;
  end;


implementation

{$R *.dfm}

constructor TImageParts.create();
begin
 inherited;
 bitmap:=TBitmap.Create();
 bitmap.PixelFormat:=pf32bit;
 //self.selected:=false;
 self.mode:=0;
 X:=0;
 Y:=0;
end;

(*IuWFNgʒu쐬*)
function TImageParts.getPosInf():TPosInf;
begin
 result.x:=self.x;
 result.y:=self.y;
 result.h:=self.h;
 result.w:=self.w;
 result.alpha:=self.alpha;
 
 if(bitmap<>nil)then
  result.trans:=self.bitmap.Transparent;

 result.mode:=self.mode;
end;

{frame}

constructor TImageFrame.create(Owner:TComponent);
begin
 inherited;
 self.drawlock:=false;

 backBitmap:=TBitmap.Create();
 initBitmap:=TBitmap.Create();
 self.backBitmap.PixelFormat:=pf32bit;
 self.backBitmap.Width:=632;
 self.backBitmap.Height:=420;

 self.arrangedBitmap:=TBitmap.Create();
 self.arrangedBitmap.PixelFormat:=pf32bit;

 self.Color:=self.backBitmap.Canvas.Brush.Color;

 self.List:=TStringList.Create();
 self.List.Sorted:=false;
 self.selectID:=-1;
 self.selectRect:=Rect(-1,-1,-1,-1);

 ADCanvas:=TADCanvas.Create();
 Image1.Canvas.Pen.Mode:=pmNot;
 Image1.Canvas.Pen.Color:=clred;
 Image1.Canvas.Brush.Color:=clgray;
 ImageWidth:=632;
 ImageHeight:=420;
 mode:=ifNormal;

end;

destructor TImageFrame.destroy();
var
 i:Integer;
begin
 arrangedBitmap.Free();
 backBitmap.Free();
 initBitmap.Free();
 ADCanvas.Free();

 for i:=0 to self.List.Count-1 do
 begin
  List.Objects[i].Free();
 end;

 inherited; 
end;

(*p[c*)
procedure TImageFrame.reset();
begin
 self.Image1.Canvas.Brush.Color:=$ffffff;
 self.Image1.Width:=632;
 self.Image1.Height:=420;
 self.selectID:=-1;
 self.Image1.Canvas.FillRect(Image1.ClientRect);
 self.initBitmap.Canvas.FillRect(initBitmap.Canvas.ClipRect);
 self.backBitmap.Assign(Image1.Picture.Bitmap);
 self.arrangedBitmap.Assign(Image1.Picture.Bitmap);
 self.List.Clear();
end;

(*p[cǉ*)
procedure TImageFrame.add(name:string;newParts:TImageParts);
var
 i:Integer;
 oldParts:TImageParts;
begin
 i:=List.IndexOf(name);

 if(i<0)then
 begin
  List.Add(name);
  List.Objects[List.count-1]:=newParts;
 end
 else
 begin
  oldparts:=TImageParts(List.Objects[i]);
  newParts.bitmap.Transparent:=oldParts.bitmap.Transparent;
  oldparts.bitmap.Handle:=newParts.bitmap.ReleaseHandle;
  oldParts.bitmap.TransparentColor:=oldParts.bitmap.Canvas.Pixels[0,0];
  newParts.Free();
 end;

 selectParts(List.count-1);

  if not(self.drawlock)then
   repaintParts();
 // else
//   selectID:=-1;

end;

(*Cӂ̃p[c̕`*)
procedure TImageFrame.drawPartsEdge(parts:TImageParts);
begin
 with parts do
  Image1.Canvas.Polyline([point(x,y),point(x+w-1,y),
   point(x+w-1,y+h-1),point(x,y+h-1),point(x,y)]);
end;

(*I𒆂̃p[cɘg`*)
procedure TImageFrame.drawSelected();
var
 oldMode:TPenMode;
begin

 Image1.Canvas.CopyRect(selectRect,backBitmap.Canvas,selectRect);
 if(selectID>-1)then
 begin
  oldMode:=Image1.Canvas.Pen.Mode;
  Image1.Canvas.Pen.Mode:=pmCopy;
  drawPartsEdge(selectedParts);

  with selectedParts do
  begin
   selectRect:=Bounds(x,y,w+1,h+1);
  end;
  Image1.Canvas.Pen.Mode:=oldMode;
 end
 else selectRect:=Rect(-1,-1,-1,-1);

end;

(*p[c폜*)
procedure TImageFrame.delete(Index:Integer);
begin
 if(index>-1)and(index<List.Count)then
 begin

  if(List.Objects[index]=selectedParts)then
  begin
   self.mode:=ifNormal;
   self.selectID:=-1;
   selectedParts:=nil;
  end;

  self.List.Objects[index].Free();
  self.List.Delete(index);
  self.repaintParts();
 end;
end;


(*p[c̏Ԃ*)
procedure TImageFrame.exchange(Index1,index2:Integer);
begin
  List.Exchange(index1,index2);
  repaintParts();
end;


(*p[cI*)
procedure TImageFrame.selectParts(index:Integer);
var
 newParts:TImageParts;
begin
 selectID:=index;
 selectedParts:=nil;

 if(index>-1)then
 begin
  newParts:=TImageParts(self.List.Objects[index]);
  selectedParts:=newParts;
 end;
  
end;

function TImageFrame.getLocation(index:Integer):TPosInf;
var
 newParts:TImageParts;
begin
 if(index>-1)then
 begin
  selectID:=index;
  newParts:=TImageParts(self.List.Objects[index]);
  selectedParts:=nil;
  selectedParts:=newParts;
  result:=selectedParts.getPosInf();
 end;
// else
 // result.selected:=false;

end;

(*Sp[cC[Wɕ`*)
procedure TImageFrame.repaintParts();
var
 i:Integer;
 newParts:TImageParts;
begin

 if not(self.drawlock)then
 begin
  Image1.Canvas.FillRect(Image1.ClientRect);
  backBitmap.Canvas.FillRect(backBitmap.Canvas.ClipRect);
  
 if(isInitImage)then
  backBitmap.Canvas.CopyRect
   (bounds(0,0,ImageWidth,ImageHeight),
    initBitmap.Canvas,initBitmap.Canvas.ClipRect);

  ADCanvas.Handle:=self.backBitmap.Canvas.Handle;

  for i:=0 to List.Count-1 do
  begin
   newparts:=TImageParts(List.Objects[i]);
   if  (newParts<>nil)then
   ADCanvas.StretchDrawEx(bounds(newparts.X,newparts.Y,
    newParts.w,newParts.h),newparts.bitmap,newParts.mode,newParts.alpha);
  end;

  Image1.Canvas.CopyRect
   (Bounds(0,0,ImageWidth,ImageHeight),backbitmap.Canvas,
    Bounds(0,0,ImageWidth,ImageHeight));

 end; 

end;

procedure TImageFrame.setInitImage(bitmap:TBitmap);
begin
 if(bitmap<>nil)then
 begin
  initBitmap.Assign(bitmap);
  initBitmap.PixelFormat:=pf32bit;
  isInitImage:=true;
 end
 else
  isInitImage:=false;

end;

procedure TImageFrame.ReadyInitImage();
var
 i:Integer;
 newParts:TImageParts;
begin

  backBitmap.Canvas.FillRect(backBitmap.Canvas.ClipRect);

 if(isInitImage)then
  backBitmap.Canvas.CopyRect
   (bounds(0,0,ImageWidth,ImageHeight),
    initBitmap.Canvas,initBitmap.Canvas.ClipRect);

  ADCanvas.Handle:=self.backBitmap.Canvas.Handle;

  for i:=0 to List.Count-1 do
  begin
   newparts:=TImageParts(List.Objects[i]);
   if  (newParts<>nil)then
   ADCanvas.StretchDrawEx(bounds(newparts.X,newparts.Y,
    newParts.w,newParts.h),newparts.bitmap,newParts.mode,newParts.alpha);
  end;

end;

function TImageFrame.checkAllParts(pt:TPoint):Integer;
var
 i:Integer;
 newParts:TImageParts;
begin
 result:=-1;
 with pt do
 begin
   for i:=List.Count-1 downto 0 do
   begin
    selectedParts:=nil;
    newParts:=TImageParts(List.Objects[i]);
    selectedParts:=newParts;
    if(x>newparts.x)and(y>newparts.y)and
       (x<newparts.x+newparts.w)and(y<newparts.y+newparts.h)then
    begin
   //  selectedParts.selected:=true;
     mypoint.X:=x;
     mypoint.Y:=y;
     result:=i;
     mode:=ifMove;
     break;
    end;
   end;
 end;

end;

procedure TImageFrame.Image1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

 if(mode=ifNormal)then
 begin

  if(selectID>-1)then
   case IsOnRect(Point(x,y),selectRect) of
    crSizeAll:
    begin
  //   selectedParts.selected:=true;
     mypoint.X:=x;
     mypoint.Y:=y;
     mode:=ifMove;
    end;
    crSizeNWSE:
    begin
  //   selectedParts.selected:=true;
     mypoint.X:=x;
     mypoint.Y:=y;
     mode:=ifReSizeAll;
    end;
    crSizeNS:
    begin
  //   selectedParts.selected:=true;
     mypoint.Y:=y;
     mypoint.X:=x;
     mode:=ifReSizeV;
    end;
    crSizeWE:
    begin
 //    selectedParts.selected:=true;
     mypoint.x:=x;
     mypoint.Y:=y;
     mode:=ifReSizeH;
    end;
    else
    begin
     selectID:=self.checkAllParts(Point(x,y));
    end
   end
  else
  begin
   selectID:=checkAllParts(Point(x,y));
  end;


   drawSelected();
   if(assigned(FNotifyPartsEvent))then
       NotifyPartsEvent(selectID,selectedParts);


 end;

end;

procedure TImageFrame.Image1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin

  if(selectID<0)then exit;

  case mode of
   ifMove:
   begin

    self.drawPartsEdge(selectedparts);

    selectedParts.x:=selectedParts.x-mypoint.X+x;
    selectedParts.y:=selectedParts.y-mypoint.Y+y;

    (*Rg[l̕ύX*)
    mypoint.X:=x;
    mypoint.Y:=y;

    self.drawPartsEdge(selectedparts);

    if(assigned(FNotifyPartsEvent))then
      NotifyPartsEvent(selectID,selectedParts);

   end;
  ifResizeAll:
  begin
    self.drawPartsEdge(selectedparts);

    selectedParts.w:=max(3,selectedParts.w-mypoint.X+x);
    selectedParts.h:=max(3,selectedParts.h-mypoint.Y+y);

    (*Rg[l̕ύX*)
    mypoint.X:=x;
    mypoint.Y:=y;

    drawPartsEdge(selectedparts);


    if(assigned(FNotifyPartsEvent))then
      NotifyPartsEvent(selectID,selectedParts);
  end;
  ifResizeV:
  begin
    self.drawPartsEdge(selectedparts);
    selectedParts.h:=max(3,selectedParts.h-mypoint.Y+y);
    mypoint.Y:=y;
    self.drawPartsEdge(selectedparts);

    if(assigned(FNotifyPartsEvent))then
      NotifyPartsEvent(selectID,selectedParts);
  end;
  ifResizeH:
  begin
    drawPartsEdge(selectedparts);
    selectedParts.w:=max(3,selectedParts.w-mypoint.X+x);
    mypoint.X:=x;
    self.drawPartsEdge(selectedparts);

    if(assigned(FNotifyPartsEvent))then
      NotifyPartsEvent(selectID,selectedParts);
  end;
  ifNormal:
   self.Cursor:=IsOnRect(Point(x,y),selectRect);
  end;

end;

procedure TImageFrame.startMove(index:Integer);
begin

end;

procedure TImageFrame.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);

begin

  case mode of
   ifMove:
   begin

    if(selectedParts.X+selectedParts.w<1)then
     selectedParts.x:=1
    else if(selectedParts.x>Image1.Width)then
     selectedParts.x:=Image1.Width-selectedParts.w;

    if(selectedParts.y+selectedParts.h<1)then
     selectedParts.y:=1
    else if(selectedParts.y>Image1.Height)then
     selectedParts.y:=Image1.Height-selectedParts.h;

    if(assigned(FNotifyPartsEvent))then
      NotifyPartsEvent(selectID,selectedParts);

  //  selectedParts.selected:=false;
    self.mode:=ifNormal;
    self.repaintParts;
    self.drawSelected();
   end;
  ifResizeAll,ifResizeV,ifResizeH:
  begin
   if(assigned(FNotifyPartsEvent))then
    NotifyPartsEvent(selectID,selectedParts);
 //  selectedParts.selected:=false;
   self.mode:=ifNormal;
   self.repaintParts;
   self.drawSelected();
  end;
 end;


end;

(*p[cɓ^*)
procedure TImageFrame.setLocation(inf:TPosInf);
var
 newParts:TImageParts;
begin
 if(self.selectID>-1)then
 begin
  newParts:=TImageParts(self.List.Objects[selectID]);
  newParts.mode:=inf.mode;
  newParts.x:=inf.x;
  newParts.y:=inf.y;
  newParts.w:=inf.w;
  newParts.h:=inf.h;
  newParts.alpha:=inf.alpha;
  newParts.bitmap.Transparent:=inf.trans;
  self.repaintParts;
 end;
end;

procedure TImageFrame.setBackColor(const color:TColor);
begin
 backBitmap.Canvas.Brush.Color:=color;
end;
             
(*摜TCY̕ύX`*)
procedure TImageFrame.resizeImage(w,h:Integer);
begin
 ImageWidth:=w;
 ImageHeight:=h;
 repaintParts();
end;

(*C[WŌ̉摜H*)
procedure TImageFrame.arrangeImage(inf:TImageInfo);
begin
 arrangedBitmap.Width:=ImageWidth;
 arrangedBitmap.Height:=ImageHeight;
 bitBlt(arrangedBitmap.Canvas.Handle,0,0,arrangedBitmap.Width,
  arrangedBitmap.Height,backBitmap.Canvas.Handle,0,0,SrcCopy);

 if(inf<>nil)then
 begin
  inc(inf.delCount);
  convImages.customizeBitmap(inf,arrangedBitmap);
  inf.FreeEx();
 end;

 if not(drawlock)then
 begin
  if(inf<>nil)and(inf.turn>0)then Image1.Canvas.FillRect(Image1.ClientRect);
  Image1.Canvas.Draw(0,0,arrangedBitmap);
 end;

end;

procedure TImageFrame.FrameDblClick(Sender: TObject);
begin

 if(selectID>-1)and(self.mode<>ifinit)then
 begin
  if(assigned(FNotifyNameEvent))then
   NotifyNameEvent(List.Strings[selectID]);
 end;

end;

(*init̔wiF擾*)
function TImageFrame.getBackColor():TColor;
begin
 result:=backBitmap.Canvas.Brush.Color;
end;

(*x[XC[Wۑ*)
procedure TImageFrame.saveImage(fileName:string;saveType:byte=1);
var
 saveBitmap:TBitmap;
begin

 saveBitmap:=TBitmap.Create();
 saveBitmap.Width:=ImageWidth;
 saveBitmap.Height:=ImageHeight;

 bitBlt(saveBitmap.Canvas.Handle,0,0,ImageWidth,ImageHeight,
  Image1.canvas.handle,0,0,srcCopy);

 if(saveType=2)then
   ClipBoard.Assign(saveBitmap)
 else
   saveBitmap.SaveToFile(fileName);

 saveBitmap.Free();

end;

(*|Cg̃p[c̒ɑ݂邩*)
function TImageFrame.IsOnRect(pt:TPoint;rec:TRect):Integer;
var
 subX,subY:Integer;
begin
 subY:=abs(pt.Y-rec.Bottom);
 subX:=abs(pt.X-rec.Right);

 if(subX<3)then
 begin
  if(subY<3)then
   result:=crSizeNWSE
  else
   result:=crSizeWE;
 end
 else if(subY<3)then
  result:=crSizeNS
 else if(pt.X>rec.Left)and(pt.X<rec.Right)and(pt.Y>rec.Top)and(pt.Y<rec.Bottom)then
  result:=crSizeAll
 else
  result:=crArrow;

end;

end.
